@@ -24,7 +24,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch -v videotestsrc ! viewfinderbin
+ * gst-launch-1.0 -v videotestsrc ! viewfinderbin
  * ]|
  * Feeds the viewfinderbin with video test data.
  * </refsect2>
@@ -57,7 +57,7 @@ enum
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw")
+    GST_STATIC_CAPS ("video/x-raw(ANY)")
     );
 
 /* class initialization */
