@@ -26,7 +26,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch -v audiotestsrc num-buffers=10 ! gstchopmydata min-size=100
+ * gst-launch-1.0 -v audiotestsrc num-buffers=10 ! chopmydata min-size=100
  * max-size=200 step-size=2 ! fakesink -v
  * ]|
  * 
@@ -284,6 +284,9 @@ gst_chop_my_data_process (GstChopMyData * chopmydata, gboolean flush)
     buffer =
         gst_adapter_take_buffer (chopmydata->adapter, chopmydata->next_size);
 
+    GST_BUFFER_PTS (buffer) = gst_adapter_prev_pts (chopmydata->adapter, NULL);
+    GST_BUFFER_DTS (buffer) = gst_adapter_prev_dts (chopmydata->adapter, NULL);
+
     chopmydata->next_size = 0;
 
     ret = gst_pad_push (chopmydata->srcpad, buffer);
