@@ -41,6 +41,20 @@
  * Boston, MA 02110-1301, USA.
  */
 
+/**
+ * SECTION:element-square
+ * @see_also: geometrictransform
+ *
+ * The square element distorts the center part of the image into a square.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch-1.0 -v videotestsrc ! square zoom=100 ! videoconvert ! autovideosink
+ * ]|
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -157,12 +171,12 @@ square_map (GstGeometricTransform * gt, gint x, gint y, gdouble * in_x,
   /* zoom at the center, smoothstep around half quadrant and get back to normal */
   norm_x *=
       (1.0 / square->zoom) * (1.0 + (square->zoom -
-          1.0) * smoothstep (square->width - 0.125, square->width + 0.125,
-          ABS (norm_x)));
+          1.0) * gst_gm_smoothstep (square->width - 0.125,
+          square->width + 0.125, ABS (norm_x)));
   norm_y *=
       (1.0 / square->zoom) * (1.0 + (square->zoom -
-          1.0) * smoothstep (square->height - 0.125, square->height + 0.125,
-          ABS (norm_y)));
+          1.0) * gst_gm_smoothstep (square->height - 0.125,
+          square->height + 0.125, ABS (norm_y)));
 
   /* unnormalize */
   *in_x = 0.5 * (norm_x + 1.0) * width;
@@ -185,16 +199,14 @@ gst_square_class_init (GstSquareClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstgt_class = (GstGeometricTransformClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gst_element_class_set_static_metadata (gstelement_class,
       "square",
       "Transform/Effect/Video",
       "Distort center part of the image into a square",
       "Filippo Argiolas <filippo.argiolas@gmail.com>");
 
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_square_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_square_get_property);
+  gobject_class->set_property = gst_square_set_property;
+  gobject_class->get_property = gst_square_get_property;
 
   g_object_class_install_property (gobject_class, PROP_WIDTH,
       g_param_spec_double ("width", "Width",
