@@ -47,6 +47,19 @@
  * his code.
  */
 
+/**
+ * SECTION:element-twirl
+ * @see_also: geometrictransform
+ *
+ * The twirl element twists the image from the center out.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * gst-launch-1.0 -v videotestsrc ! twirl ! videoconvert ! autovideosink
+ * ]|
+ * </refsect2>
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -116,13 +129,6 @@ gst_twirl_get_property (GObject * object, guint prop_id,
   }
 }
 
-/* Clean up */
-static void
-gst_twirl_finalize (GObject * obj)
-{
-  G_OBJECT_CLASS (parent_class)->finalize (obj);
-}
-
 static gboolean
 twirl_map (GstGeometricTransform * gt, gint x, gint y, gdouble * in_x,
     gdouble * in_y)
@@ -165,17 +171,14 @@ gst_twirl_class_init (GstTwirlClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstgt_class = (GstGeometricTransformClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gst_element_class_set_static_metadata (gstelement_class,
       "twirl",
       "Transform/Effect/Video",
       "Twists the image from the center out",
       "Thiago Santos<thiago.sousa.santos@collabora.co.uk>");
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_twirl_finalize);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_twirl_set_property);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_twirl_get_property);
+  gobject_class->set_property = gst_twirl_set_property;
+  gobject_class->get_property = gst_twirl_get_property;
 
   g_object_class_install_property (gobject_class, PROP_ANGLE,
       g_param_spec_double ("angle", "angle",
