@@ -2114,7 +2114,7 @@ gst_ahc_src_setcaps (GstBaseSrc * src, GstCaps * caps)
       }
     }
     gst_ahc_parameters_supported_preview_fps_range_free (ranges);
-    if (self->fps_max == 0) {
+    if (self->fps_max == 0 || self->fps_min == 0) {
       GST_ERROR_OBJECT (self, "Couldn't find an applicable FPS range");
       goto end;
     }
@@ -2494,6 +2494,10 @@ gst_ahc_src_query (GstBaseSrc * bsrc, GstQuery * query)
     case GST_QUERY_LATENCY:{
       GstClockTime min;
 
+      /* Cannot query latency before setcaps() */
+      if (self->fps_min == 0)
+        return FALSE;
+
       /* Allow of 1 frame latency base on the longer frame duration */
       gst_query_parse_latency (query, NULL, &min, NULL);
       min = gst_util_uint64_scale (GST_SECOND, 1000, self->fps_min);
