@@ -2193,8 +2193,10 @@ gst_amc_video_dec_handle_frame (GstVideoDecoder * decoder,
     /* Copy the buffer content in chunks of size as requested
      * by the port */
     buf = gst_amc_codec_get_input_buffer (self->codec, idx, &err);
-    if (!buf)
+    if (err)
       goto failed_to_get_input_buffer;
+    else if (!buf)
+      goto got_null_input_buffer;
 
     memset (&buffer_info, 0, sizeof (buffer_info));
     buffer_info.offset = 0;
@@ -2268,6 +2270,15 @@ failed_to_get_input_buffer:
     gst_video_codec_frame_unref (frame);
     return GST_FLOW_ERROR;
   }
+got_null_input_buffer:
+  {
+    GST_ELEMENT_ERROR (self, LIBRARY, SETTINGS, (NULL),
+        ("Got no input buffer"));
+    if (minfo.data)
+      gst_buffer_unmap (frame->input_buffer, &minfo);
+    gst_video_codec_frame_unref (frame);
+    return GST_FLOW_ERROR;
+  }
 dequeue_error:
   {
     GST_ELEMENT_ERROR_FROM_ERROR (self, err);
