@@ -157,10 +157,11 @@ static void
 gst_av_sample_video_sink_finalize (GObject * object)
 {
   GstAVSampleVideoSink *av_sink = GST_AV_SAMPLE_VIDEO_SINK (object);
+  __block AVSampleBufferDisplayLayer *layer = av_sink->layer;
 
-  if (av_sink->layer) {
+  if (layer) {
     dispatch_async (dispatch_get_main_queue (), ^{
-      [av_sink->layer release];
+      [layer release];
     });
   }
 
@@ -748,11 +749,13 @@ gst_av_sample_video_sink_show_frame (GstVideoSink * vsink, GstBuffer * buf)
     _request_data (av_sink);
   g_mutex_unlock (&av_sink->render_lock);
 
+#if defined(MAC_OS_X_VERSION_MAX_ALLOWED) && MAC_OS_X_VERSION_MAX_ALLOWED >= 1010
   if ([av_sink->layer status] == AVQueuedSampleBufferRenderingStatusFailed) {
     GST_ERROR_OBJECT (av_sink, "failed to enqueue buffer on layer, %s",
         [[[av_sink->layer error] description] UTF8String]);
     return GST_FLOW_ERROR;
   }
+#endif
 
   return ret;
 }
