@@ -51,12 +51,13 @@
 #include <mmreg.h>
 #include <dmoreg.h>
 #include <wmcodecdsp.h>
+#include <gst/audio/audio.h>
 
 GST_DEBUG_CATEGORY_STATIC (dshowaudiodec_debug);
 #define GST_CAT_DEFAULT dshowaudiodec_debug
 
-GST_BOILERPLATE (GstDshowAudioDec, gst_dshowaudiodec, GstElement,
-    GST_TYPE_ELEMENT);
+#define gst_dshowaudiodec_parent_class parent_class
+G_DEFINE_TYPE(GstDshowAudioDec, gst_dshowaudiodec, GST_TYPE_ELEMENT)
 
 static void gst_dshowaudiodec_finalize (GObject * object);
 static GstStateChangeReturn gst_dshowaudiodec_change_state
@@ -64,8 +65,8 @@ static GstStateChangeReturn gst_dshowaudiodec_change_state
 
 /* sink pad overwrites */
 static gboolean gst_dshowaudiodec_sink_setcaps (GstPad * pad, GstCaps * caps);
-static GstFlowReturn gst_dshowaudiodec_chain (GstPad * pad, GstBuffer * buffer);
-static gboolean gst_dshowaudiodec_sink_event (GstPad * pad, GstEvent * event);
+static GstFlowReturn gst_dshowaudiodec_chain (GstPad * pad, GstObject *parent, GstBuffer * buffer);
+static gboolean gst_dshowaudiodec_sink_event (GstPad * pad, GstObject *parent, GstEvent * event);
 
 /* utils */
 static gboolean gst_dshowaudiodec_create_graph_and_filters (GstDshowAudioDec *
@@ -158,7 +159,7 @@ HRESULT AudioFakeSink::DoRenderSample(IMediaSample *pMediaSample)
   GstBuffer *out_buf = NULL;
   gboolean in_seg = FALSE;
   GstClockTime buf_start, buf_stop;
-  gint64 clip_start = 0, clip_stop = 0;
+  guint64 clip_start = 0, clip_stop = 0;
   guint start_offset = 0, stop_offset;
   GstClockTime duration;
 
@@ -199,20 +200,21 @@ HRESULT AudioFakeSink::DoRenderSample(IMediaSample *pMediaSample)
      * GstBuffer for output, and clip if required */
 
     /* allocate a new buffer for raw audio */
-    mDec->last_ret = gst_pad_alloc_buffer (mDec->srcpad, 
-        GST_BUFFER_OFFSET_NONE,
-        size,
-        GST_PAD_CAPS (mDec->srcpad), &out_buf);
+    out_buf = gst_buffer_new_and_alloc(size);
     if (!out_buf) {
       GST_WARNING_OBJECT (mDec, "cannot allocate a new GstBuffer");
       goto done;
     }
-
+    
     /* set buffer properties */
     GST_BUFFER_TIMESTAMP (out_buf) = buf_start;
     GST_BUFFER_DURATION (out_buf) = duration;
-    memcpy (GST_BUFFER_DATA (out_buf), pBuffer,
-        MIN ((unsigned int)size, GST_BUFFER_SIZE (out_buf)));
+
+    if (gst_buffer_fill(out_buf, 0, pBuffer, size) != size) {
+      gst_buffer_unref (out_buf);
+      GST_WARNING_OBJECT (mDec, "unable to fill output buffer");
+      goto done;
+    }
 
     /* we have to remove some heading samples */
     if ((GstClockTime) clip_start > buf_start) {
@@ -231,11 +233,11 @@ HRESULT AudioFakeSink::DoRenderSample(IMediaSample *pMediaSample)
 
     /* truncating */
     if ((start_offset != 0) || (stop_offset != (size_t) size)) {
-      GstBuffer *subbuf = gst_buffer_create_sub (out_buf, start_offset,
-          stop_offset - start_offset);
+      
+      GstBuffer *subbuf = gst_buffer_copy_region (out_buf, GST_BUFFER_COPY_ALL, 
+        start_offset, stop_offset - start_offset);
 
       if (subbuf) {
-        gst_buffer_set_caps (subbuf, GST_PAD_CAPS (mDec->srcpad));
         gst_buffer_unref (out_buf);
         out_buf = subbuf;
       }
@@ -305,6 +307,21 @@ HRESULT AudioFakeSink::CheckMediaType(const CMediaType *pmt)
   return S_FALSE;
 }
 
+int AudioFakeSink::GetBufferSize()
+{
+  IMemAllocator *allocator = NULL;
+  if (m_pInputPin) {
+    allocator = m_pInputPin->Allocator();
+    if(allocator) {
+      ALLOCATOR_PROPERTIES props;
+      allocator->GetProperties(&props);
+      return props.cbBuffer;
+    }
+  }
+
+  return 0;
+}
+
 static void
 gst_dshowaudiodec_base_init (gpointer klass)
 {
@@ -312,36 +329,34 @@ gst_dshowaudiodec_base_init (gpointer klass)
   GstPadTemplate *src, *sink;
   GstCaps *srccaps, *sinkcaps;
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  GstElementDetails details;
   const AudioCodecEntry *tmp;
   gpointer qdata;
+  gchar *longname, *description;
 
   qdata = g_type_get_qdata (G_OBJECT_CLASS_TYPE (klass), DSHOW_CODEC_QDATA);
 
   /* element details */
   tmp = audiodec_class->entry = (AudioCodecEntry *) qdata;
 
-  details.longname = g_strdup_printf ("DirectShow %s Decoder Wrapper",
+  longname = g_strdup_printf ("DirectShow %s Decoder Wrapper",
       tmp->element_longname);
-  details.klass = g_strdup ("Codec/Decoder/Audio");
-  details.description = g_strdup_printf ("DirectShow %s Decoder Wrapper",
+  description = g_strdup_printf ("DirectShow %s Decoder Wrapper",
       tmp->element_longname);
-  details.author = "Sebastien Moutte <sebastien@moutte.net>";
-  gst_element_class_set_details (element_class, &details);
-  g_free (details.longname);
-  g_free (details.klass);
-  g_free (details.description);
+
+  gst_element_class_set_metadata(element_class, longname, "Codec/Decoder/Audio", description, 
+    "Sebastien Moutte <sebastien@moutte.net>");
+  
+  g_free (longname);
+  g_free (description);
 
   sinkcaps = gst_caps_from_string (tmp->sinkcaps);
 
   srccaps = gst_caps_from_string (
-      "audio/x-raw-int,"
-      "width = (int)[1, 32],"
-      "depth = (int)[1, 32],"
+      "audio/x-raw,"
+      "format = (string)" GST_AUDIO_FORMATS_ALL ","
       "rate = (int)[1, MAX],"
       "channels = (int)[1, MAX],"
-      "signed = (boolean)true,"
-      "endianness = (int)" G_STRINGIFY(G_LITTLE_ENDIAN));
+      "layout = (string)interleaved");
 
   sink = gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS, sinkcaps);
   src = gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS, srccaps);
@@ -370,7 +385,7 @@ gst_dshowaudiodec_com_thread (GstDshowAudioDec * adec)
 {
   HRESULT res;
 
-  g_mutex_lock (adec->com_init_lock);
+  g_mutex_lock (&adec->com_init_lock);
 
   /* Initialize COM with a MTA for this process. This thread will
    * be the first one to enter the apartement and the last one to leave
@@ -387,24 +402,23 @@ gst_dshowaudiodec_com_thread (GstDshowAudioDec * adec)
   adec->comInitialized = TRUE;
 
   /* Signal other threads waiting on this condition that COM was initialized */
-  g_cond_signal (adec->com_initialized);
+  g_cond_signal (&adec->com_initialized);
 
-  g_mutex_unlock (adec->com_init_lock);
+  g_mutex_unlock (&adec->com_init_lock);
 
   /* Wait until the unitialize condition is met to leave the COM apartement */
-  g_mutex_lock (adec->com_deinit_lock);
-  g_cond_wait (adec->com_uninitialize, adec->com_deinit_lock);
+  g_mutex_lock (&adec->com_deinit_lock);
+  g_cond_wait (&adec->com_uninitialize, &adec->com_deinit_lock);
 
   CoUninitialize ();
   GST_INFO_OBJECT (adec, "COM unintialized succesfully");
   adec->comInitialized = FALSE;
-  g_cond_signal (adec->com_uninitialized);
-  g_mutex_unlock (adec->com_deinit_lock);
+  g_cond_signal (&adec->com_uninitialized);
+  g_mutex_unlock (&adec->com_deinit_lock);
 }
 
 static void
-gst_dshowaudiodec_init (GstDshowAudioDec * adec,
-    GstDshowAudioDecClass * adec_class)
+gst_dshowaudiodec_init (GstDshowAudioDec * adec)
 {
   GstElementClass *element_class = GST_ELEMENT_GET_CLASS (adec);
 
@@ -413,7 +427,6 @@ gst_dshowaudiodec_init (GstDshowAudioDec * adec,
       gst_pad_new_from_template (gst_element_class_get_pad_template
       (element_class, "sink"), "sink");
 
-  gst_pad_set_setcaps_function (adec->sinkpad, gst_dshowaudiodec_sink_setcaps);
   gst_pad_set_event_function (adec->sinkpad, gst_dshowaudiodec_sink_event);
   gst_pad_set_chain_function (adec->sinkpad, gst_dshowaudiodec_chain);
   gst_element_add_pad (GST_ELEMENT (adec), adec->sinkpad);
@@ -443,21 +456,21 @@ gst_dshowaudiodec_init (GstDshowAudioDec * adec,
 
   adec->last_ret = GST_FLOW_OK;
 
-  adec->com_init_lock = g_mutex_new();
-  adec->com_deinit_lock = g_mutex_new();
-  adec->com_initialized = g_cond_new();
-  adec->com_uninitialize = g_cond_new();
-  adec->com_uninitialized = g_cond_new();
+  g_mutex_init(&adec->com_init_lock);
+  g_mutex_init(&adec->com_deinit_lock);
+  g_cond_init(&adec->com_initialized);
+  g_cond_init(&adec->com_uninitialize);
+  g_cond_init(&adec->com_uninitialized);
 
-  g_mutex_lock (adec->com_init_lock);
+  g_mutex_lock (&adec->com_init_lock);
 
   /* create the COM initialization thread */
-  g_thread_create ((GThreadFunc)gst_dshowaudiodec_com_thread,
-      adec, FALSE, NULL);
+  g_thread_new ("COM init thread", (GThreadFunc)gst_dshowaudiodec_com_thread, 
+    adec);
 
   /* wait until the COM thread signals that COM has been initialized */
-  g_cond_wait (adec->com_initialized, adec->com_init_lock);
-  g_mutex_unlock (adec->com_init_lock);
+  g_cond_wait (&adec->com_initialized, &adec->com_init_lock);
+  g_mutex_unlock (&adec->com_init_lock);
 }
 
 static void
@@ -477,17 +490,17 @@ gst_dshowaudiodec_finalize (GObject * object)
 
   /* signal the COM thread that it sould uninitialize COM */
   if (adec->comInitialized) {
-    g_mutex_lock (adec->com_deinit_lock);
-    g_cond_signal (adec->com_uninitialize);
-    g_cond_wait (adec->com_uninitialized, adec->com_deinit_lock);
-    g_mutex_unlock (adec->com_deinit_lock);
+    g_mutex_lock (&adec->com_deinit_lock);
+    g_cond_signal (&adec->com_uninitialize);
+    g_cond_wait (&adec->com_uninitialized, &adec->com_deinit_lock);
+    g_mutex_unlock (&adec->com_deinit_lock);
   }
 
-  g_mutex_free (adec->com_init_lock);
-  g_mutex_free (adec->com_deinit_lock);
-  g_cond_free (adec->com_initialized);
-  g_cond_free (adec->com_uninitialize);
-  g_cond_free (adec->com_uninitialized);
+  g_mutex_clear (&adec->com_init_lock);
+  g_mutex_clear (&adec->com_deinit_lock);
+  g_cond_clear (&adec->com_initialized);
+  g_cond_clear (&adec->com_uninitialize);
+  g_cond_clear (&adec->com_uninitialized);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -529,7 +542,7 @@ gst_dshowaudiodec_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  return GST_ELEMENT_CLASS(parent_class)->change_state (element, transition);
 }
 
 static gboolean
@@ -571,9 +584,10 @@ end:
 }
 
 static GstFlowReturn
-gst_dshowaudiodec_chain (GstPad * pad, GstBuffer * buffer)
+gst_dshowaudiodec_chain (GstPad *pad, GstObject *parent, GstBuffer *buffer)
 {
   GstDshowAudioDec *adec = (GstDshowAudioDec *) gst_pad_get_parent (pad);
+  GstMapInfo map;
   bool discont = FALSE;
 
   if (!adec->setup) {
@@ -583,7 +597,7 @@ gst_dshowaudiodec_chain (GstPad * pad, GstBuffer * buffer)
     goto beach;
   }
 
-  if (GST_FLOW_IS_FATAL (adec->last_ret)) {
+  if (adec->last_ret != GST_FLOW_OK) {
     GST_DEBUG_OBJECT (adec, "last decoding iteration generated a fatal error "
         "%s", gst_flow_get_name (adec->last_ret));
     goto beach;
@@ -591,7 +605,7 @@ gst_dshowaudiodec_chain (GstPad * pad, GstBuffer * buffer)
 
   GST_CAT_DEBUG_OBJECT (dshowaudiodec_debug, adec, "chain (size %d)=> pts %"
       GST_TIME_FORMAT " stop %" GST_TIME_FORMAT,
-      GST_BUFFER_SIZE (buffer), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
+      gst_buffer_get_size(buffer), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer) +
           GST_BUFFER_DURATION (buffer)));
 
@@ -605,10 +619,12 @@ gst_dshowaudiodec_chain (GstPad * pad, GstBuffer * buffer)
   }
 
   /* push the buffer to the directshow decoder */
+  gst_buffer_map(buffer, &map, GST_MAP_READ);
   adec->fakesrc->GetOutputPin()->PushBuffer (
-      GST_BUFFER_DATA (buffer), GST_BUFFER_TIMESTAMP (buffer),
+      map.data, GST_BUFFER_TIMESTAMP (buffer),
       GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer),
-      GST_BUFFER_SIZE (buffer), (bool)discont);
+      map.size, (bool)discont);
+  gst_buffer_unmap(buffer, &map);
 
 beach:
   gst_buffer_unref (buffer);
@@ -617,51 +633,45 @@ beach:
 }
 
 static gboolean
-gst_dshowaudiodec_sink_event (GstPad * pad, GstEvent * event)
+gst_dshowaudiodec_sink_event (GstPad * pad, GstObject *parent, GstEvent * event)
 {
   gboolean ret = TRUE;
-  GstDshowAudioDec *adec = (GstDshowAudioDec *) gst_pad_get_parent (pad);
+  GstDshowAudioDec *adec = (GstDshowAudioDec *) parent;
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_FLUSH_STOP:{
-      gst_dshowaudiodec_flush (adec);
-      ret = gst_pad_event_default (pad, event);
+    case GST_EVENT_CAPS:{
+      GstCaps *caps;
+      gst_event_parse_caps(event, &caps);
+      ret = gst_dshowaudiodec_sink_setcaps(pad, caps);
       break;
     }
-    case GST_EVENT_NEWSEGMENT:
-    {
-      GstFormat format;
-      gdouble rate;
-      gint64 start, stop, time;
-      gboolean update;
 
-      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
-          &stop, &time);
+    case GST_EVENT_FLUSH_STOP:{
+      gst_dshowaudiodec_flush (adec);
+      ret = gst_pad_event_default (pad, parent, event);
+      break;
+    }
+
+    case GST_EVENT_SEGMENT:{
+      const GstSegment *segment;
+      gst_event_parse_segment (event, &segment);
 
       GST_CAT_DEBUG_OBJECT (dshowaudiodec_debug, adec,
           "received new segment from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (start), GST_TIME_ARGS (stop));
-
-      if (update) {
-        GST_CAT_DEBUG_OBJECT (dshowaudiodec_debug, adec,
-            "closing current segment flushing..");
-        gst_dshowaudiodec_flush (adec);
-      }
+          GST_TIME_ARGS (segment->start), GST_TIME_ARGS (segment->stop));
 
       /* save the new segment in our local current segment */
-      gst_segment_set_newsegment (adec->segment, update, rate, format, start,
-          stop, time);
+      gst_segment_copy_into(segment, adec->segment);
 
-      ret = gst_pad_event_default (pad, event);
+      ret = gst_pad_event_default (pad, parent, event);
       break;
     }
+
     default:
-      ret = gst_pad_event_default (pad, event);
+      ret = gst_pad_event_default (pad, parent, event);
       break;
   }
 
-  gst_object_unref (adec);
-
   return ret;
 }
 
@@ -757,7 +767,7 @@ dshowaudiodec_set_input_format (GstDshowAudioDec *adec, GstCaps *caps)
   else 
   {
     size = sizeof (WAVEFORMATEX) +
-        (adec->codec_data ? GST_BUFFER_SIZE (adec->codec_data) : 0);
+        (adec->codec_data ? gst_buffer_get_size(adec->codec_data) : 0);
 
     if (adec->layer == 3) {
       MPEGLAYER3WAVEFORMAT *mp3format;
@@ -783,11 +793,12 @@ dshowaudiodec_set_input_format (GstDshowAudioDec *adec, GstCaps *caps)
     }
     else {
       format = (WAVEFORMATEX *)g_malloc0 (size);
+
       if (adec->codec_data) {     /* Codec data is appended after our header */
-        memcpy (((guchar *) format) + sizeof (WAVEFORMATEX),
-            GST_BUFFER_DATA (adec->codec_data),
-            GST_BUFFER_SIZE (adec->codec_data));
-        format->cbSize = GST_BUFFER_SIZE (adec->codec_data);
+        gsize codec_size = gst_buffer_get_size(adec->codec_data);
+        gst_buffer_extract(adec->codec_data, 0, ((guchar *) format) + sizeof (WAVEFORMATEX), 
+          codec_size);
+        format->cbSize = codec_size;
       }
     }
 
@@ -843,8 +854,7 @@ dshowaudiodec_set_output_format (GstDshowAudioDec *adec)
 static void
 dshowadec_free_mediatype (AM_MEDIA_TYPE *mediatype)
 {
-  if (mediatype->pbFormat)
-    g_free (mediatype->pbFormat);
+  g_free (mediatype->pbFormat);
   g_free (mediatype);
 }
 
@@ -855,14 +865,15 @@ gst_dshowaudiodec_setup_graph (GstDshowAudioDec * adec, GstCaps *caps)
   GstDshowAudioDecClass *klass =
       (GstDshowAudioDecClass *) G_OBJECT_GET_CLASS (adec);
   HRESULT hres;
-  GstCaps *outcaps;
+  GstCaps *outcaps = NULL;
   AM_MEDIA_TYPE *output_mediatype = NULL;
   AM_MEDIA_TYPE *input_mediatype = NULL;
-  CComPtr<IPin> output_pin;
-  CComPtr<IPin> input_pin;
+  IPinPtr output_pin = NULL;
+  IPinPtr input_pin = NULL;
   const AudioCodecEntry *codec_entry = klass->entry;
-  CComQIPtr<IBaseFilter> srcfilter;
-  CComQIPtr<IBaseFilter> sinkfilter;
+  IBaseFilterPtr srcfilter;
+  IBaseFilterPtr sinkfilter;
+  GstAudioInfo audio_info;
 
   input_mediatype = dshowaudiodec_set_input_format (adec, caps);
 
@@ -901,22 +912,18 @@ gst_dshowaudiodec_setup_graph (GstDshowAudioDec * adec, GstCaps *caps)
 
   adec->fakesink->SetMediaType(output_mediatype);
 
-  outcaps = gst_caps_new_simple ("audio/x-raw-int",
-      "width", G_TYPE_INT, adec->depth,
-      "depth", G_TYPE_INT, adec->depth,
-      "rate", G_TYPE_INT, adec->rate,
-      "channels", G_TYPE_INT, adec->channels, 
-      "signed", G_TYPE_BOOLEAN, TRUE,
-      "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
-      NULL);
+  gst_audio_info_init(&audio_info);
+  gst_audio_info_set_format(&audio_info, 
+    gst_audio_format_build_integer(TRUE, G_BYTE_ORDER, adec->depth, adec->depth),
+    adec->rate, adec->channels, NULL);
+
+  outcaps = gst_audio_info_to_caps(&audio_info);
 
   if (!gst_pad_set_caps (adec->srcpad, outcaps)) {
-    gst_caps_unref (outcaps);
     GST_ELEMENT_ERROR (adec, CORE, NEGOTIATION,
         ("Failed to negotiate output"), (NULL));
     goto end;
   }
-  gst_caps_unref (outcaps);
 
   /* connect the decoder to our fake sink */
   output_pin = gst_dshow_get_pin_from_filter (adec->decfilter, PINDIR_OUTPUT);
@@ -951,6 +958,8 @@ gst_dshowaudiodec_setup_graph (GstDshowAudioDec * adec, GstCaps *caps)
   ret = TRUE;
   adec->setup = TRUE;
 end:
+  if (outcaps)
+    gst_caps_unref(outcaps);
   if (input_mediatype)
     dshowadec_free_mediatype (input_mediatype);
   if (output_mediatype)
@@ -962,8 +971,8 @@ end:
 static gboolean
 gst_dshowaudiodec_get_filter_settings (GstDshowAudioDec * adec)
 {
-  CComPtr<IPin> output_pin;
-  CComPtr<IEnumMediaTypes> enum_mediatypes;
+  IPinPtr output_pin;
+  IEnumMediaTypesPtr enum_mediatypes;
   HRESULT hres;
   ULONG fetched;
   BOOL ret = FALSE;
@@ -1008,13 +1017,13 @@ gst_dshowaudiodec_create_graph_and_filters (GstDshowAudioDec * adec)
   HRESULT hres;
   GstDshowAudioDecClass *klass =
       (GstDshowAudioDecClass *) G_OBJECT_GET_CLASS (adec);
-  CComQIPtr<IBaseFilter> srcfilter;
-  CComQIPtr<IBaseFilter> sinkfilter;
+  IBaseFilterPtr srcfilter;
+  IBaseFilterPtr sinkfilter;
   GUID insubtype = GUID_MEDIASUBTYPE_FROM_FOURCC (klass->entry->format);
   GUID outsubtype = GUID_MEDIASUBTYPE_FROM_FOURCC (WAVE_FORMAT_PCM);
 
   /* create the filter graph manager object */
-  hres = adec->filtergraph.CoCreateInstance (
+  hres = adec->filtergraph.CreateInstance (
       CLSID_FilterGraph, NULL, CLSCTX_INPROC);
   if (FAILED (hres)) {
     GST_ELEMENT_ERROR (adec, STREAM, FAILED,
@@ -1102,7 +1111,7 @@ gst_dshowaudiodec_destroy_graph_and_filters (GstDshowAudioDec * adec)
 
   if (adec->fakesrc) {
     if (adec->filtergraph) {
-      CComQIPtr<IBaseFilter> filter = adec->fakesrc;
+      IBaseFilterPtr filter = adec->fakesrc;
       adec->filtergraph->RemoveFilter(filter);
     }
     adec->fakesrc->Release();
@@ -1115,7 +1124,7 @@ gst_dshowaudiodec_destroy_graph_and_filters (GstDshowAudioDec * adec)
   }
   if (adec->fakesink) {
     if (adec->filtergraph) {
-      CComQIPtr<IBaseFilter> filter = adec->fakesink;
+      IBaseFilterPtr filter = adec->fakesink;
       adec->filtergraph->RemoveFilter(filter);
     }
 
@@ -1153,7 +1162,7 @@ dshow_adec_register (GstPlugin * plugin)
   hr = CoInitialize(0);
   for (i = 0; i < sizeof (audio_dec_codecs) / sizeof (AudioCodecEntry); i++) {
     GType type;
-    CComPtr<IBaseFilter> filter;
+    IBaseFilterPtr filter;
     GUID insubtype = GUID_MEDIASUBTYPE_FROM_FOURCC (audio_dec_codecs[i].format);
     GUID outsubtype = GUID_MEDIASUBTYPE_FROM_FOURCC (WAVE_FORMAT_PCM);
 
